import type { ThemedToken, ThemeRegistration } from '@shikijs/types'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

const baseStyles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
  },
  codeContainer: {
    padding: 16,
    minWidth: '100%',
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
  token: {
    fontFamily: monospaceFont,
    fontSize: 14,
  },
})

interface Props extends SyntaxHighlighterProps {
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const SyntaxHighlighter: React.FC<Props> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId: languageId || String(language),
    theme,
  })

  const themeStyles = useMemo(() => getRNStylesFromShikiStyle(theme as ThemeRegistration), [theme])

  const renderToken = (token: ThemedToken, rowIndex: number, tokenIndex: number) => {
    const tokenStyle: TextStyle = {
      color: token.color || themeStyles.editor?.color,
      fontStyle: token.fontStyle ? 'italic' : 'normal',
      fontWeight: token.fontStyle ? 'bold' : 'normal',
    }

    return (
      <Text key={`${rowIndex}-${tokenIndex}`} style={[baseStyles.token, tokenStyle, textStyle, { fontSize }]}>
        {token.content}
      </Text>
    )
  }

  return (
    <ScrollView
      horizontal
      {...scrollViewProps}
      style={[baseStyles.scrollView, themeStyles.editor, scrollViewProps?.style]}
      contentContainerStyle={[baseStyles.codeContainer, scrollViewProps?.contentContainerStyle, containerStyle]}
    >
      <View onStartShouldSetResponder={() => true}>
        {tokens.map((line, rowIndex) => (
          <View key={rowIndex} style={baseStyles.lineContainer}>
            {line.map((token, tokenIndex) => renderToken(token, rowIndex, tokenIndex))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
